@@ -12,6 +12,8 @@ from trytond.backend import TableHandler
 from trytond.pyson import Eval, If, Bool
 from trytond.transaction import Transaction
 from trytond.pool import Pool
+from trytond.modules.calendar import AlarmMixin, DateMixin, RRuleMixin, \
+    AttendeeMixin
 
 __all__ = ['Todo', 'TodoCategory', 'TodoRDate', 'TodoRRule', 'TodoExDate',
     'TodoExRule', 'TodoAttendee', 'TodoAlarm']
@@ -117,13 +119,10 @@ class Todo(ModelSQL, ModelView):
         cls._sql_constraints = [
             #XXX should be unique across all componenets
             ('uuid_recurrence_uniq', 'UNIQUE(uuid, calendar, recurrence)',
-                'UUID and recurrence must be unique in a calendar!'),
-            ]
-        cls._constraints += [
-            ('check_recurrence', 'invalid_recurrence'),
+                'UUID and recurrence must be unique in a calendar.'),
             ]
         cls._error_messages.update({
-                'invalid_recurrence': 'Recurrence can not be recurrent!',
+                'invalid_recurrence': 'Todo "%s" can not be recurrent.',
                 })
 
     @classmethod
@@ -136,7 +135,6 @@ class Todo(ModelSQL, ModelView):
             models_data = ModelData.search([
                 ('fs_id', '=', 'rule_group_read_todo_line3'),
                 ('module', '=', module_name),
-                ('inherit', '=', None),
                 ], limit=1)
             if models_data:
                 model_data, = models_data
@@ -191,6 +189,12 @@ class Todo(ModelSQL, ModelView):
     def search_calendar_field(cls, name, clause):
         return [('calendar.' + name[9:],) + tuple(clause[1:])]
 
+    @classmethod
+    def validate(cls, todos):
+        super(Todo, cls).validate(todos)
+        for todo in todos:
+            todo.check_recurrence()
+
     def check_recurrence(self):
         '''
         Check the recurrence is not recurrent.
@@ -202,63 +206,64 @@ class Todo(ModelSQL, ModelView):
                 or self.exdates
                 or self.exrules
                 or self.occurences):
-            return False
-        return True
+            self.raise_user_error('invalid_recurrence', (self.rec_name,))
 
     @classmethod
-    def create(cls, values):
+    def create(cls, vlist):
         pool = Pool()
         Calendar = pool.get('calendar.calendar')
         Collection = pool.get('webdav.collection')
 
-        todo = super(Todo, cls).create(values)
-        if (todo.calendar.owner
-                and (todo.organizer == todo.calendar.owner.email
-                    or (todo.parent
-                        and todo.parent.organizer == \
-                            todo.parent.calendar.owner.email))):
-            if todo.organizer == todo.calendar.owner.email:
-                attendee_emails = [x.email for x in todo.attendees
-                        if x.status != 'declined'
-                        and x.email != todo.organizer]
-            else:
-                attendee_emails = [x.email for x in todo.parent.attendees
-                        if x.status != 'declined'
-                        and x.email != todo.parent.organizer]
-            if attendee_emails:
-                with Transaction().set_user(0):
-                    calendars = Calendar.search([
-                        ('owner.email', 'in', attendee_emails),
-                        ])
-                    if not todo.recurrence:
-                        for calendar in calendars:
-                            new_todo, = cls.copy([todo], default={
-                                'calendar': calendar.id,
-                                'occurences': None,
-                                'uuid': todo.uuid,
-                                })
-                            for occurence in todo.occurences:
-                                cls.copy([occurence], default={
-                                    'calendar': calendar.id,
-                                    'parent': new_todo.id,
-                                    'uuid': occurence.uuid,
-                                    })
-                    else:
-                        parents = cls.search([
-                            ('uuid', '=', todo.uuid),
-                            ('calendar.owner.email', 'in', attendee_emails),
-                            ('id', '!=', todo.id),
-                            ('recurrence', '=', None),
+        todos = super(Todo, cls).create(vlist)
+        for todo in todos:
+            if (todo.calendar.owner
+                    and (todo.organizer == todo.calendar.owner.email
+                        or (todo.parent
+                            and todo.parent.organizer
+                            == todo.parent.calendar.owner.email))):
+                if todo.organizer == todo.calendar.owner.email:
+                    attendee_emails = [x.email for x in todo.attendees
+                            if x.status != 'declined'
+                            and x.email != todo.organizer]
+                else:
+                    attendee_emails = [x.email for x in todo.parent.attendees
+                            if x.status != 'declined'
+                            and x.email != todo.parent.organizer]
+                if attendee_emails:
+                    with Transaction().set_user(0):
+                        calendars = Calendar.search([
+                            ('owner.email', 'in', attendee_emails),
                             ])
-                        for parent in parents:
-                            cls.copy([todo], default={
-                                'calendar': parent.calendar.id,
-                                'parent': parent.id,
-                                'uuid': todo.uuid,
-                                })
+                        if not todo.recurrence:
+                            for calendar in calendars:
+                                new_todo, = cls.copy([todo], default={
+                                    'calendar': calendar.id,
+                                    'occurences': None,
+                                    'uuid': todo.uuid,
+                                    })
+                                for occurence in todo.occurences:
+                                    cls.copy([occurence], default={
+                                        'calendar': calendar.id,
+                                        'parent': new_todo.id,
+                                        'uuid': occurence.uuid,
+                                        })
+                        else:
+                            parents = cls.search([
+                                    ('uuid', '=', todo.uuid),
+                                    ('calendar.owner.email', 'in',
+                                        attendee_emails),
+                                    ('id', '!=', todo.id),
+                                    ('recurrence', '=', None),
+                                    ])
+                            for parent in parents:
+                                cls.copy([todo], default={
+                                    'calendar': parent.calendar.id,
+                                    'parent': parent.id,
+                                    'uuid': todo.uuid,
+                                    })
         # Restart the cache for todo
         Collection._todo_cache.clear()
-        return todo
+        return todos
 
     def _todo2update(self):
         res = {}
@@ -271,21 +276,21 @@ class Todo(ModelSQL, ModelView):
         res['status'] = self.status
         res['organizer'] = self.organizer
         res['rdates'] = [('delete_all',)]
-        for rdate in self.rdates:
-            vals = rdate._date2update()
-            res['rdates'].append(('create', vals))
+        to_create = [rdate._date2update() for rdate in self.rdates]
+        if to_create:
+            res['rdates'].append(('create', to_create))
         res['exdates'] = [('delete_all',)]
-        for exdate in self.exdates:
-            vals = exdate._date2update()
-            res['exdates'].append(('create', vals))
+        to_create = [exdate._date2update() for exdate in self.exdates]
+        if to_create:
+            res['exdates'].append(('create', to_create))
         res['rrules'] = [('delete_all',)]
-        for rrule in self.rrules:
-            vals = rrule._rule2update()
-            res['rrules'].append(('create', vals))
+        to_create = [rrule._rule2update() for rrule in self.rrules]
+        if to_create:
+            res['rrules'].append(('create', to_create))
         res['exrules'] = [('delete_all',)]
-        for exrule in self.exrules:
-            vals = exrule._rule2update()
-            res['exrules'].append(('create', vals))
+        to_create = [exrule._rule2update() for exrule in self.exrules]
+        if to_create:
+            res['exrules'].append(('create', to_create))
         return res
 
     @classmethod
@@ -306,15 +311,16 @@ class Todo(ModelSQL, ModelView):
         for i in range(0, len(ids), cursor.IN_MAX):
             sub_ids = ids[i:i + cursor.IN_MAX]
             red_sql, red_ids = reduce_ids('id', sub_ids)
-            cursor.execute('UPDATE "' + cls._table + '" ' \
-                    'SET sequence = sequence + 1 ' \
-                    'WHERE ' + red_sql, red_ids)
+            cursor.execute('UPDATE "' + cls._table + '" '
+                'SET sequence = sequence + 1 '
+                'WHERE ' + red_sql, red_ids)
 
         for todo in todos:
-            if todo.calendar.owner \
-                    and (todo.organizer == todo.calendar.owner.email \
-                    or (todo.parent \
-                    and todo.parent.organizer == todo.calendar.owner.email)):
+            if (todo.calendar.owner
+                    and (todo.organizer == todo.calendar.owner.email
+                        or (todo.parent
+                            and todo.parent.organizer
+                            == todo.calendar.owner.email))):
                 if todo.organizer == todo.calendar.owner.email:
                     attendee_emails = [x.email for x in todo.attendees
                             if x.status != 'declined'
@@ -378,10 +384,11 @@ class Todo(ModelSQL, ModelView):
         Collection = pool.get('webdav.collection')
 
         for todo in todos:
-            if todo.calendar.owner \
-                    and (todo.organizer == todo.calendar.owner.email \
-                    or (todo.parent \
-                    and todo.parent.organizer == todo.calendar.owner.email)):
+            if (todo.calendar.owner
+                    and (todo.organizer == todo.calendar.owner.email
+                        or (todo.parent
+                            and todo.parent.organizer
+                            == todo.calendar.owner.email))):
                 if todo.organizer == todo.calendar.owner.email:
                     attendee_emails = [x.email for x in todo.attendees
                             if x.email != todo.organizer]
@@ -522,7 +529,7 @@ class Todo(ModelSQL, ModelView):
             else:
                 if vtodo.recurrence_id.value.tzinfo:
                     res['recurrence'] = \
-                            vtodo.recurrence_id.value.astimezone(tzlocal)
+                        vtodo.recurrence_id.value.astimezone(tzlocal)
                 else:
                     res['recurrence'] = vtodo.recurrence_id.value
         else:
@@ -538,11 +545,14 @@ class Todo(ModelSQL, ModelView):
             category_names2ids = {}
             for category in categories:
                 category_names2ids[category.name] = category.id
+            to_create = []
             for category in vtodo.categories.value:
                 if category not in category_names2ids:
-                    categories.append(Category.create({
-                        'name': category,
-                        }))
+                    to_create.append({
+                            'name': category,
+                            })
+            if to_create:
+                categories += Category.create(to_create)
             res['categories'] = [('set', [c.id for c in categories])]
         else:
             res['categories'] = [('unlink_all',)]
@@ -559,9 +569,9 @@ class Todo(ModelSQL, ModelView):
                 ('name', '=', vtodo.location.value),
                 ], limit=1)
             if not locations:
-                location, = Location.create({
-                    'name': vtodo.location.value,
-                    })
+                location, = Location.create([{
+                            'name': vtodo.location.value,
+                            }])
             else:
                 location, = locations
             res['location'] = location.id
@@ -584,6 +594,7 @@ class Todo(ModelSQL, ModelView):
                 attendees_todel[attendee.email] = attendee.id
         res['attendees'] = []
         if hasattr(vtodo, 'attendee'):
+            to_create = []
             while vtodo.attendee_list:
                 attendee = vtodo.attendee_list.pop()
                 vals = Attendee.attendee2values(attendee)
@@ -592,56 +603,67 @@ class Todo(ModelSQL, ModelView):
                         attendees_todel[vals['email']], vals))
                     del attendees_todel[vals['email']]
                 else:
-                    res['attendees'].append(('create', vals))
+                    to_create.append(vals)
+            if to_create:
+                res['attendees'].append(('create', to_create))
         res['attendees'].append(('delete', attendees_todel.values()))
 
         res['rdates'] = []
         if todo:
             res['rdates'].append(('delete', [x.id for x in todo.rdates]))
         if hasattr(vtodo, 'rdate'):
+            to_create = []
             while vtodo.rdate_list:
                 rdate = vtodo.rdate_list.pop()
-                for date in rdate.value:
-                    vals = Rdate.date2values(date)
-                    res['rdates'].append(('create', vals))
+                to_create += [Rdate.date2values(date) for date in rdate.value]
+            if to_create:
+                res['rdates'].append(('create', to_create))
 
         res['exdates'] = []
         if todo:
             res['exdates'].append(('delete', [x.id for x in todo.exdates]))
         if hasattr(vtodo, 'exdate'):
+            to_create = []
             while vtodo.exdate_list:
                 exdate = vtodo.exdate_list.pop()
-                for date in exdate.value:
-                    vals = Exdate.date2values(date)
-                    res['exdates'].append(('create', vals))
+                to_create += [Exdate.date2values(date)
+                    for date in exdate.value]
+            if to_create:
+                res['exdates'].append(('create', to_create))
 
         res['rrules'] = []
         if todo:
             res['rrules'].append(('delete', [x.id for x in todo.rrules]))
         if hasattr(vtodo, 'rrule'):
+            to_create = []
             while vtodo.rrule_list:
                 rrule = vtodo.rrule_list.pop()
-                vals = Rrule.rule2values(rrule)
-                res['rrules'].append(('create', vals))
+                to_create.append(Rrule.rule2values(rrule))
+            if to_create:
+                res['rrules'].append(('create', to_create))
 
         res['exrules'] = []
         if todo:
             res['exrules'].append(('delete', [x.id for x in todo.exrules]))
         if hasattr(vtodo, 'exrule'):
+            to_create = []
             while vtodo.exrule_list:
                 exrule = vtodo.exrule_list.pop()
-                vals = Exrule.rule2values(exrule)
-                res['exrules'].append(('create', vals))
+                to_create.append(Exrule.rule2values(exrule))
+            if to_create:
+                res['exrules'].append(('create', to_create))
 
         if todo:
             res.setdefault('alarms', [])
             res['alarms'].append(('delete', [x.id for x in todo.alarms]))
         if hasattr(vtodo, 'valarm'):
             res.setdefault('alarms', [])
+            to_create = []
             while vtodo.valarm_list:
                 valarm = vtodo.valarm_list.pop()
-                vals = Alarm.valarm2values(valarm)
-                res['alarms'].append(('create', vals))
+                to_create.append(Alarm.valarm2values(valarm))
+            if to_create:
+                res['alarms'].append(('create', to_create))
 
         if hasattr(ical, 'vtimezone'):
             if ical.vtimezone.tzid.value in pytz.common_timezones:
@@ -656,6 +678,7 @@ class Todo(ModelSQL, ModelView):
         occurences_todel = []
         if todo:
             occurences_todel = [x.id for x in todo.occurences]
+        to_create = []
         for vtodo in vtodos:
             todo_id = None
             if todo:
@@ -673,7 +696,9 @@ class Todo(ModelSQL, ModelView):
             if todo_id:
                 res['occurences'].append(('write', todo_id, vals))
             else:
-                res['occurences'].append(('create', vals))
+                to_create.append(vals)
+        if to_create:
+            res['occurences'].append(('create', to_create))
         if occurences_todel:
             res.setdefault('occurences', [])
             res['occurences'].append(('delete', occurences_todel))
@@ -723,7 +748,7 @@ class Todo(ModelSQL, ModelView):
             if not hasattr(vtodo, 'completed'):
                 vtodo.add('completed')
             vtodo.completed.value = self.completed.replace(tzinfo=tzlocal)\
-                    .astimezone(tzutc)
+                .astimezone(tzutc)
         elif hasattr(vtodo, 'completed'):
             del vtodo.completed
 
@@ -731,7 +756,7 @@ class Todo(ModelSQL, ModelView):
             if not hasattr(vtodo, 'dtstart'):
                 vtodo.add('dtstart')
             vtodo.dtstart.value = self.dtstart.replace(tzinfo=tzlocal)\
-                    .astimezone(tztodo)
+                .astimezone(tztodo)
         elif hasattr(vtodo, 'dtstart'):
             del vtodo.dtstart
 
@@ -739,7 +764,7 @@ class Todo(ModelSQL, ModelView):
             if not hasattr(vtodo, 'due'):
                 vtodo.add('due')
             vtodo.due.value = self.due.replace(tzinfo=tzlocal)\
-                    .astimezone(tztodo)
+                .astimezone(tztodo)
         elif hasattr(vtodo, 'due'):
             del vtodo.due
 
@@ -759,7 +784,7 @@ class Todo(ModelSQL, ModelView):
             if not hasattr(vtodo, 'recurrence-id'):
                 vtodo.add('recurrence-id')
             vtodo.recurrence_id.value = self.recurrence\
-                    .replace(tzinfo=tzlocal).astimezone(tztodo)
+                .replace(tzinfo=tzlocal).astimezone(tztodo)
         elif hasattr(vtodo, 'recurrence-id'):
             del vtodo.recurrence_id
         if self.status:
@@ -845,13 +870,10 @@ class TodoCategory(ModelSQL):
             ondelete='CASCADE', required=True, select=True)
 
 
-class TodoRDate(ModelSQL, ModelView):
+class TodoRDate(DateMixin, ModelSQL, ModelView):
     'Todo Recurrence Date'
     __name__ = 'calendar.todo.rdate'
-    _inherits = {'calendar.date': 'calendar_date'}
     _rec_name = 'datetime'
-    calendar_date = fields.Many2One('calendar.date', 'Calendar Date',
-            required=True, ondelete='CASCADE', select=True)
     todo = fields.Many2One('calendar.todo', 'Todo', ondelete='CASCADE',
             select=True, required=True)
 
@@ -866,13 +888,30 @@ class TodoRDate(ModelSQL, ModelView):
 
         super(TodoRDate, cls).__register__(module_name)
 
+        table = TableHandler(cursor, cls, module_name)
+
+        # Migration from 2.6: Remove inherits calendar.date
+        if table.column_exist('calendar_date'):
+            cursor.execute('UPDATE "' + cls._table + '" AS e '
+                'SET date = (SELECT a.date '
+                    'FROM calendar_date AS a '
+                    'WHERE a.id = e.calendar_date), '
+                'datetime = (SELECT a.datetime '
+                    'FROM calendar_date AS a '
+                    'WHERE a.id = e.calendar_date)')
+            table.drop_column('calendar_date', True)
+
     @classmethod
-    def create(cls, values):
+    def create(cls, vlist):
         Todo = Pool().get('calendar.todo')
-        if values.get('todo'):
-            # Update write_date of todo
-            Todo.write([Todo(values['todo'])], {})
-        return super(TodoRDate, cls).create(values)
+        towrite = []
+        for values in vlist:
+            if values.get('todo'):
+                # Update write_date of todo
+                towrite.append(values['todo'])
+        if towrite:
+            Todo.write(Todo.browse(towrite), {})
+        return super(TodoRDate, cls).create(vlist)
 
     @classmethod
     def write(cls, rdates, values):
@@ -889,47 +928,50 @@ class TodoRDate(ModelSQL, ModelView):
     def delete(cls, todo_rdates):
         pool = Pool()
         Todo = pool.get('calendar.todo')
-        Rdate = pool.get('calendar.date')
-        rdates = [a.calendar_date for a in todo_rdates]
         todos = [x.todo for x in todo_rdates]
         if todos:
             # Update write_date of todo
             Todo.write(todos, {})
         super(TodoRDate, cls).delete(todo_rdates)
-        if rdates:
-            Rdate.delete(rdates)
-
-    def _date2update(self):
-        return self.calendar_date._date2update()
-
-    @classmethod
-    def date2values(cls, date):
-        Date = Pool().get('calendar.date')
-        return Date.date2values(date.calendar_date)
-
-    @classmethod
-    def date2date(cls, date):
-        Date = Pool().get('calendar.date')
-        return Date.date2date(date.calendar_date)
 
 
-class TodoRRule(ModelSQL, ModelView):
+class TodoRRule(RRuleMixin, ModelSQL, ModelView):
     'Recurrence Rule'
     __name__ = 'calendar.todo.rrule'
-    _inherits = {'calendar.rrule': 'calendar_rrule'}
     _rec_name = 'freq'
-    calendar_rrule = fields.Many2One('calendar.rrule', 'Calendar RRule',
-            required=True, ondelete='CASCADE', select=True)
     todo = fields.Many2One('calendar.todo', 'Todo', ondelete='CASCADE',
             select=True, required=True)
 
     @classmethod
-    def create(cls, values):
+    def __register__(cls, module_name):
+        cursor = Transaction().cursor
+
+        super(TodoRRule, cls).__register__(module_name)
+
+        table = TableHandler(cursor, cls, module_name)
+
+        # Migration from 2.6: Remove inherits calendar.rrule
+        if table.column_exist('calendar_rrule'):
+            for field in (f for f in dir(RRuleMixin)
+                    if isinstance(f, fields.Field)):
+                cursor.execute(('UPDATE "' + cls._table + '" AS e '
+                        'SET "%(field)s" = (SELECT a."%(field)s" '
+                            'FROM calendar_rrule AS r '
+                            'WHERE r.id = e.calendar_rrule)')
+                    % {'field': field})
+            table.drop_column('calendar_rrule', True)
+
+    @classmethod
+    def create(cls, vlist):
         Todo = Pool().get('calendar.todo')
-        if values.get('todo'):
-            # Update write_date of todo
-            Todo.write([Todo(values['todo'])], {})
-        return super(TodoRRule, cls).create(values)
+        towrite = []
+        for values in vlist:
+            if values.get('todo'):
+                # Update write_date of todo
+                towrite.append(values['todo'])
+        if towrite:
+            Todo.write(Todo.browse(towrite), {})
+        return super(TodoRRule, cls).create(vlist)
 
     @classmethod
     def write(cls, todo_rrules, values):
@@ -946,28 +988,11 @@ class TodoRRule(ModelSQL, ModelView):
     def delete(cls, todo_rrules):
         pool = Pool()
         Todo = pool.get('calendar.todo')
-        Rrule = pool.get('calendar.rrule')
-        rrules = [a.calendar_rrule for a in todo_rrules]
         todos = [x.todo for x in todo_rrules]
         if todos:
             # Update write_date of todo
             Todo.write(todos, {})
         super(TodoRRule, cls).delete(todo_rrules)
-        if rrules:
-            Rrule.delete(rrules)
-
-    def _rule2update(self):
-        return self.calendar_rrule._rule2update()
-
-    @classmethod
-    def rule2values(cls, rule):
-        Rule = Pool().get('calendar.rrule')
-        return Rule.rule2values(rule.calendar_rrule)
-
-    @classmethod
-    def rule2rule(cls, rule):
-        Rule = Pool().get('calendar.rrule')
-        return Rule.rule2rule(rule.calendar_rrule)
 
 
 class TodoExDate(TodoRDate):
@@ -982,48 +1007,69 @@ class TodoExRule(TodoRRule):
     _table = 'calendar_todo_exrule'  # Needed to override TodoRRule._table
 
 
-class TodoAttendee(ModelSQL, ModelView):
+class TodoAttendee(AttendeeMixin, ModelSQL, ModelView):
     'Attendee'
     __name__ = 'calendar.todo.attendee'
-    _inherits = {'calendar.attendee': 'calendar_attendee'}
-    calendar_attendee = fields.Many2One('calendar.attendee',
-        'Calendar Attendee', required=True, ondelete='CASCADE', select=True)
     todo = fields.Many2One('calendar.todo', 'Todo', ondelete='CASCADE',
             required=True, select=True)
 
     @classmethod
-    def create(cls, values):
+    def __register__(cls, module_name):
+        cursor = Transaction().cursor
+
+        super(TodoAttendee, cls).__register__(module_name)
+
+        table = TableHandler(cursor, cls, module_name)
+
+        # Migration from 2.6: Remove inherits calendar.attendee
+        if table.column_exist('calendar_attendee'):
+            cursor.execute('UPDATE "' + cls._table + '" AS e '
+                'SET email = (SELECT a.email '
+                    'FROM calendar_attendee AS a '
+                    'WHERE a.id = e.calendar_attendee), '
+                'status = (SELECT a.status '
+                    'FROM calendar_attendee AS a '
+                    'WHERE a.id = e.calendar_attendee)')
+            table.drop_column('calendar_attendee', True)
+
+    @classmethod
+    def create(cls, vlist):
         Todo = Pool().get('calendar.todo')
 
-        if values.get('todo'):
-            # Update write_date of todo
-            Todo.write([Todo(values['todo'])], {})
-        attendee = super(TodoAttendee, cls).create(values)
-        todo = attendee.todo
-        if (todo.calendar.owner
-                and (todo.organizer == todo.calendar.owner.email
-                    or (todo.parent
-                        and todo.parent.organizer == \
-                            todo.parent.calendar.owner.email))):
-            if todo.organizer == todo.calendar.owner.email:
-                attendee_emails = [x.email for x in todo.attendees
-                        if x.email != todo.organizer]
-            else:
-                attendee_emails = [x.email for x in todo.parent.attendees
-                        if x.email != todo.parent.organizer]
-            if attendee_emails:
-                with Transaction().set_user(0):
-                    todos = Todo.search([
-                        ('uuid', '=', todo.uuid),
-                        ('calendar.owner.email', 'in', attendee_emails),
-                        ('id', '!=', todo.id),
-                        ('recurrence', '=', todo.recurrence),
-                        ])
-                    for todo in todos:
-                        cls.copy([attendee], default={
-                            'todo': todo.id,
-                            })
-        return attendee
+        towrite = []
+        for values in vlist:
+            if values.get('todo'):
+                # Update write_date of todo
+                towrite.append(values['todo'])
+        if towrite:
+            Todo.write(Todo.browse(towrite), {})
+        attendees = super(TodoAttendee, cls).create(vlist)
+        for attendee in attendees:
+            todo = attendee.todo
+            if (todo.calendar.owner
+                    and (todo.organizer == todo.calendar.owner.email
+                        or (todo.parent
+                            and todo.parent.organizer
+                            == todo.parent.calendar.owner.email))):
+                if todo.organizer == todo.calendar.owner.email:
+                    attendee_emails = [x.email for x in todo.attendees
+                            if x.email != todo.organizer]
+                else:
+                    attendee_emails = [x.email for x in todo.parent.attendees
+                            if x.email != todo.parent.organizer]
+                if attendee_emails:
+                    with Transaction().set_user(0):
+                        todos = Todo.search([
+                            ('uuid', '=', todo.uuid),
+                            ('calendar.owner.email', 'in', attendee_emails),
+                            ('id', '!=', todo.id),
+                            ('recurrence', '=', todo.recurrence),
+                            ])
+                        for todo in todos:
+                            cls.copy([attendee], default={
+                                'todo': todo.id,
+                                })
+        return attendees
 
     @classmethod
     def write(cls, attendees, values):
@@ -1043,10 +1089,11 @@ class TodoAttendee(ModelSQL, ModelView):
         super(TodoAttendee, cls).write(attendees, values)
         for attendee in attendees:
             todo = attendee.todo
-            if todo.calendar.owner \
-                    and (todo.organizer == todo.calendar.owner.email \
-                    or (todo.parent \
-                    and todo.parent.organizer == todo.calendar.owner.email)):
+            if (todo.calendar.owner
+                    and (todo.organizer == todo.calendar.owner.email
+                        or (todo.parent
+                            and todo.parent.organizer
+                            == todo.calendar.owner.email))):
                 if todo.organizer == todo.calendar.owner.email:
                     attendee_emails = [x.email for x in todo.attendees
                             if x.email != todo.organizer]
@@ -1056,22 +1103,20 @@ class TodoAttendee(ModelSQL, ModelView):
                 if attendee_emails:
                     with Transaction().set_user(0):
                         attendees2 = cls.search([
-                            ('todo.uuid', '=', todo.uuid),
-                            ('todo.calendar.owner.email', 'in',
+                                ('todo.uuid', '=', todo.uuid),
+                                ('todo.calendar.owner.email', 'in',
                                     attendee_emails),
-                            ('id', '!=', attendee.id),
-                            ('todo.recurrence', '=', todo.recurrence),
-                            ('email', '=', attendee.email),
-                            ])
+                                ('id', '!=', attendee.id),
+                                ('todo.recurrence', '=', todo.recurrence),
+                                ('email', '=', attendee.email),
+                                ])
                         cls.write(attendees2, attendee._attendee2update())
 
     @classmethod
     def delete(cls, todo_attendees):
         pool = Pool()
         Todo = pool.get('calendar.todo')
-        Attendee = pool.get('calendar.attendee')
 
-        calendar_attendees = [a.calendar_attendee for a in todo_attendees]
         todos = [x.todo for x in todo_attendees]
         if todos:
             # Update write_date of todo
@@ -1079,10 +1124,11 @@ class TodoAttendee(ModelSQL, ModelView):
 
         for attendee in todo_attendees:
             todo = attendee.todo
-            if todo.calendar.owner \
-                    and (todo.organizer == todo.calendar.owner.email \
-                    or (todo.parent \
-                    and todo.parent.organizer == todo.calendar.owner.email)):
+            if (todo.calendar.owner
+                    and (todo.organizer == todo.calendar.owner.email
+                        or (todo.parent
+                            and todo.parent.organizer
+                            == todo.calendar.owner.email))):
                 if todo.organizer == todo.calendar.owner.email:
                     attendee_emails = [x.email for x in todo.attendees
                             if x.email != todo.organizer]
@@ -1100,10 +1146,10 @@ class TodoAttendee(ModelSQL, ModelView):
                             ('email', '=', attendee.email),
                             ])
                         cls.delete(attendees)
-            elif todo.calendar.organizer \
-                    and ((todo.organizer \
-                    or (todo.parent and todo.parent.organizer)) \
-                    and attendee.email == todo.calendar.owner.email):
+            elif (todo.calendar.organizer
+                    and ((todo.organizer
+                            or (todo.parent and todo.parent.organizer))
+                        and attendee.email == todo.calendar.owner.email)):
                 if todo.organizer:
                     organizer = todo.organizer
                 else:
@@ -1121,55 +1167,41 @@ class TodoAttendee(ModelSQL, ModelView):
                             'status': 'declined',
                             })
         super(TodoAttendee, cls).delete(todo_attendees)
-        if calendar_attendees:
-            Attendee.delete(calendar_attendees)
-
-    @classmethod
-    def copy(cls, todo_attendees, default=None):
-        Attendee = Pool().get('calendar.attendee')
-
-        if default is None:
-            default = {}
-        default = default.copy()
-        new_attendees = []
-        for attendee in todo_attendees:
-            default['calendar_attendee'], = Attendee.copy(
-                [attendee.calendar_attendee])
-            new_attendee, = super(TodoAttendee, cls).copy([attendee],
-                default=default)
-            new_attendees.append(new_attendee)
-        return new_attendees
-
-    def _attendee2update(self):
-        return self.calendar_attendee._attendee2update()
-
-    @classmethod
-    def attendee2values(cls, attendee):
-        Attendee = Pool().get('calendar.attendee')
-        return Attendee.attendee2values(attendee.calendar_attendee)
-
-    @classmethod
-    def attendee2attendee(cls, attendee):
-        Attendee = Pool().get('calendar.attendee')
-        return Attendee.attendee2attendee(attendee.calendar_attendee)
 
 
-class TodoAlarm(ModelSQL):
+class TodoAlarm(AlarmMixin, ModelSQL, ModelView):
     'Alarm'
     __name__ = 'calendar.todo.alarm'
-    _inherits = {'calendar.alarm': 'calendar_alarm'}
-    calendar_alarm = fields.Many2One('calendar.alarm', 'Calendar Alarm',
-            required=True, ondelete='CASCADE', select=True)
     todo = fields.Many2One('calendar.todo', 'Todo', ondelete='CASCADE',
             required=True, select=True)
 
     @classmethod
-    def create(cls, values):
+    def __register__(cls, module_name):
+        cursor = Transaction().cursor
+
+        super(TodoAlarm, cls).__register__(module_name)
+
+        table = TableHandler(cursor, cls, module_name)
+
+        # Migration from 2.6: Remove inherits calendar.alarm
+        if table.column_exist('calendar_alarm'):
+            cursor.execute('UPDATE "' + cls._table + '" AS t '
+                'SET valarm = (SELECT a.valarm '
+                    'FROM calendar_alarm AS a '
+                    'WHERE a.id = t.calendar_alarm)')
+            table.drop_column('calendar_alarm', True)
+
+    @classmethod
+    def create(cls, vlist):
         Todo = Pool().get('calendar.todo')
-        if values.get('todo'):
-            # Update write_date of todo
-            Todo.write([Todo(values['todo'])], {})
-        return super(TodoAlarm, cls).create(values)
+        towrite = []
+        for values in vlist:
+            if values.get('todo'):
+                # Update write_date of todo
+                towrite.append(values['todo'])
+        if towrite:
+            Todo.write(Todo.browse(towrite), {})
+        return super(TodoAlarm, cls).create(vlist)
 
     @classmethod
     def write(cls, alarms, values):
@@ -1186,21 +1218,8 @@ class TodoAlarm(ModelSQL):
     def delete(cls, todo_alarms):
         pool = Pool()
         Todo = pool.get('calendar.todo')
-        Alarm = pool.get('calendar.alarm')
-        alarms = [a.calendar_alarm for a in todo_alarms]
         todos = [x.todo for x in todo_alarms]
         if todos:
             # Update write_date of todo
             Todo.write(todos, {})
         super(TodoAlarm, cls).delete(todo_alarms)
-        if alarms:
-            Alarm.delete(alarms)
-
-    @classmethod
-    def valarm2values(cls, alarm):
-        Alarm = Pool().get('calendar.alarm')
-        return Alarm.valarm2values(alarm.calendar_alarm)
-
-    def alarm2valarm(self, alarm):
-        Alarm = Pool().get('calendar.alarm')
-        return Alarm.alarm2valarm(alarm.calendar_alarm)
